/* $Workfile: B8DEMO.C $       $Revision: 31 $      $Modtime: 6/23/98 7:46p $  */
/*------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1994-1997

  NAME

    B8DEMO.C

  DESCRIPTION

    Example program which demonstrates the use of the API for
    the Balder-8S 2.0 board.

\*--------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include <ctype.h>

#include "balder.h"              /* Balder Standard Application Programming Interface */
#include "bldsys.h"

#include "logf.h"
#include "cmds.h"


/*--------------------------------------------------------------------------*\
  MACROS
\*--------------------------------------------------------------------------*/


#ifdef BLD2
  #define BALDER_BOARD_TYPE  122         /* Unique code for Balder-2 2.0    */
  #define DEFAULT_LOG_FILE "bld2demo.log"
#else
  #define BALDER_BOARD_TYPE  128         /* Unique code for Balder-8 2.0    */
  #define DEFAULT_LOG_FILE "bld8demo.log"
#endif


#define DEFAULT_DSP_APP_FILE "..\\..\\dtmffsk.out"



/*--------------------------------------------------------------------------*\
  TYPES
\*--------------------------------------------------------------------------*/
typedef struct envT {
  short irq;
  unsigned long baseAddr;
  short debugLog;
  char  *logFileName;
  char  *dspAppFile;
  short allLiTeMode;
  short allLiNtMode;
  short evenLiNtMode;
  short skip2;
  short autoActivate;
  BldLawType law;
} envT;




/*--------------------------------------------------------------------------*\
  GLOBAL VARIABLES
\*--------------------------------------------------------------------------*/
envT env;


/*--------------------------------------------------------------------------*\
  PRIVATE VARIABLES
\*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*\
  PRIVATE FUNCTION PROTOTYPES
\*--------------------------------------------------------------------------*/
static MyBool installDriver(unsigned long baseAddr, short irq);
static void helpPanel(void);
static void parseCmdLineArgs(int argc, char *argv[], envT *env);



/*--------------------------------------------------------------------------*\

  SYNOPSIS:

    This is where it all starts.

\*--------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
  short i;

  printf("B8DEMO, %s, Copyright (c) Odin TeleSystems Inc. 1994-1997\n", __DATE__);
  printf("Using Driver: %s\n\n", bldIdentDriver());

  // Defaults
  env.irq          = 10;
  env.baseAddr     = 0x280;
  env.debugLog     = FALSE;
  env.logFileName  = DEFAULT_LOG_FILE;
  env.dspAppFile   = DEFAULT_DSP_APP_FILE;
  env.allLiTeMode  = FALSE;
  env.allLiNtMode  = FALSE;
  env.evenLiNtMode = FALSE;
  env.skip2        = FALSE;
  env.autoActivate = FALSE;
  env.law          = LT_ALAW;

  parseCmdLineArgs(argc, argv, &env);

  if ((logF = fopen(env.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", env.logFileName);
    exit(-1);
  }

  fprintf(logF, "\n/*-------------------------------------------------------*\\\n");
  fprintf(logF, "B8DEMO, %s, Copyright (c) Odin TeleSystems Inc. 1994-1997\n", __DATE__);
  fprintf(logF, "Using Driver: %s\n", bldIdentDriver());
  for (i = 0; i < argc; i++) {
    fprintf(logF, "%s ", argv[i]);
  }
  fprintf(logF, "\n");

  logf("Using IRQ=%d\n", env.irq);
  logf("Using IoBaseAddr=0x%.X\n", env.baseAddr);
  logf("Log file is '%s'\n", env.logFileName);
  fprintf(logF, "\n\\*-------------------------------------------------------*/\n\n");

  if (!installDriver(env.baseAddr, env.irq)) {
    cmdsExit(1);
  }

  logf("Press '?' for help, 'q' to quit\n");

  cmdsFromKeyboard();

  logf("Normal program termination...\n");

  cmdsExit(0);

  return 0;

}







static MyBool installDriver(
/*---------------------------------------------------- Function Description -*\

  SYNOPSIS

    Sets up the driver by initializing the software with the
    proper IRQs and such, and then all LI are configured.

  RETURNS

    TRUE   // No error(s)
    FALSE  // Error(s)

\*---------------------------------------------------------------------------*/
  unsigned long baseAddr,
  short irq
)
{
  BldRc rc; // Return code from BLD functions
  short liNo;
  short seriousError = FALSE;

  if ((rc = bldConstructDriver(BALDER_BOARD_TYPE, baseAddr, NO_OF_BOARDS, irq)) != BLD_SUCCESS) {
    logBldRc(rc);
    return FALSE;
  }

  if ((rc = bldInitDriver()) != BLD_SUCCESS) {
    logBldRc(rc);
    return FALSE;
  }


  for (liNo=0; liNo <= LI_MAX; liNo++) {
    LiMode liMode[8];
    /* Configure a line interface.                         */
    /* for generation we always configure all LIs as TEs   */

    bldTermLi(liNo, TRUE); // Connect terminating resistor


#ifdef BLD2
    //liMode = (liNo % 2 == 0) ? BLD_TE : BLD_LT_S;
    liMode = BLD_TE;
#else
    if (env.allLiTeMode) {
      liMode[liNo] = BLD_TE;
    } else if (env.allLiNtMode) {
      liMode[liNo] = BLD_LT_S;
    } else {
      if (env.evenLiNtMode) {
        liMode[liNo] = (liNo % 2 == 1) ? BLD_TE : BLD_LT_S;
      } else {
        liMode[liNo] = (liNo % 2 == 0) ? BLD_TE : BLD_LT_S;
      }
    }
    #if 1 // Fix for 9708401 board
    if (env.skip2) {
      if (liNo == 2) continue;
    }
    #endif
#endif

    logf("Configuring Li#%d in %s mode\n", liNo, (liMode[liNo] == BLD_TE) ? "TE" : "NT");

    if ((rc = bldConfigureLi(liNo, liMode[liNo])) != BLD_SUCCESS) {
      if (rc == BLD_L1_DOWN) {
        logf("Li:%d  Configured OK, but Layer 1 is inactive\n", liNo);
      } else {
        logBldRc(rc);
        if (env.irq != BLD_NO_IRQ) {
          seriousError = TRUE;
        }
      }
    }

    #if 1 // (JARI, 12/10/97)
    if (env.autoActivate) {
      // activate immediately ...
      long timeout = 500;
      if (liMode[liNo] == BLD_LT_S) {
        timeout = 5000;
      }
      if ((rc = bldActivate(liNo, timeout)) != BLD_SUCCESS) {
         logBldRc(rc);
      }
    }
    #endif


  }


#if 0 // This is done in B8Boot now
  {
    short liNo;

    for (liNo = 0; liNo <= LI_MAX; liNo++) {
      logf("Loading DSP Application %s to DSP#%d (%c-law) \n", env.dspAppFile, liNo, env.law == LT_ALAW ? 'A' : 'U');
      if ((rc = bldConfigureDsp(liNo, env.dspAppFile, "_c_int00", env.law)) != BLD_SUCCESS) {
        logBldRc(rc);
        seriousError = TRUE;
      }
      //demoDelay(100L);
    }
  }
#else
  {
    short liNo;

    for (liNo = 0; liNo <= LI_MAX; liNo++) {
      logf("Reseting DSP#%d (%c-law) \n", liNo, env.law == LT_ALAW ? 'A' : 'U');
      if ((rc = bldRestartDsp(liNo, env.law)) != BLD_SUCCESS) {
        logBldRc(rc);
        seriousError = TRUE;
      }
    }
  }
#endif



#if 0 // For continous test....
  if (seriousError) {
    cmdsExit(1);
  } else {
    cmdsExit(0);
  }
#endif

  return !seriousError;  // No Errors but L1 may be down, who knows...
}









static void parseCmdLineArgs(
/*---------------------------------------------------- Function Description -*\
  SYNOPSIS

    Parse the programs command line arguments and calls upon requested
    actions.

    There are some optional switches that can be used following the
    driver name on the command line. Switches are are separated by
    spaces, and can appear in arbitrary order. Switches consist of a
    hyphen and a letter:

\*------------------------------------------------------------------------*/
  int argc,
  char *argv[],
  envT *env
)
{
  short  i;

  for (i = 1; i < argc; i++) {        // Scan command line arguments
    strupr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'A') {          // IO-address
        env->baseAddr = (unsigned long) strtoul(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'I') {   // Irq
        env->irq = (short) strtol(argv[i]+2, NULL, 0);
      } else if (*(argv[i]+1) == 'L') {   // Log file
        env->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'F') {   // DSP App File
        env->dspAppFile = argv[i]+2;
      } else if (*(argv[i]+1) == 'C') {   // Companding Law
        if (*(argv[i]+2) == 'A') {   // A-law
          env->law = LT_ALAW;
        } else if (*(argv[i]+2) == 'U') {   // U-law
          env->law = LT_ULAW;
        } else {
          printf("Error: Invalid companding law command line switch %s\n", argv[i]);
          helpPanel();
          exit(1);
        }
      } else if (*(argv[i]+1) == 'D') {   // DSP Debugger mode
        env->debugLog = TRUE;
      } else if (*(argv[i]+1) == 'T') {   // Run All Lis in TE mode
        env->allLiTeMode = TRUE;
      } else if (*(argv[i]+1) == 'N') {   // Run All Lis in TE mode
        env->allLiNtMode = TRUE;
      } else if (*(argv[i]+1) == 'E') {   // Run Even Lis in NT-mode
        env->evenLiNtMode = TRUE;

      } else if (*(argv[i]+1) == 'P') {   // Run in polling mode
        env->irq = BLD_NO_IRQ;

      } else if (*(argv[i]+1) == 'U') {   // Automatic activation of links
        env->autoActivate = TRUE;

      } else if (*(argv[i]+1) == '2') {
        env->skip2 = TRUE;

      } else {
        printf("Error: Unknown command line switch %s\n", argv[i]);
        helpPanel();
        exit(1);
      }
    } else {
      printf("Error: Unknown command line switch %s\n", argv[i]);
      helpPanel();
      exit(1);
    }
  }

}

static void helpPanel(
/*---------------------------------------------------- Function Description -*\

  SYNOPSIS

    Prints available command line switches to the console.

\*---------------------------------------------------------------------------*/
  void
)
{
  printf("\nCommand line switches:\n");
  printf(" /A<address>  : IoBaseAddress\n");
  printf(" /I<irq>      : Irq\n");
  printf(" /L<fileName> : Log file name\n");
  printf(" /F<fileName> : DSP Application File to be loaded at startup\n");
  printf(" /D           : Write Debug Messages to the log file as well\n");
  printf("                 (NOTE: Only Safe in Polling Mode (/P option) )\n");
  printf(" /T           : All 8 Lis configured in TE-mode (overrides /E switch)\n");
  printf(" /N           : All 8 Lis configured in NT-mode (overrides /E switch)\n");
  printf(" /E           : Even Lis configured in NT-mode\n");
  printf(" /P           : Run in polling mode\n");
  printf(" /U           : Automatic Link Activation\n");
  printf(" /C<a|u>      : Companding law (A-law or U-law)\n");
}




/* $History: B8DEMO.C $
 * 
 * *****************  Version 31  *****************
 * User: Hanz         Date: 6/24/98    Time: 8:48a
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Apps/B8Demo
 * Just fixed a compiler warning.
 * 
 * *****************  Version 30  *****************
 * User: Hanz         Date: 6/23/98    Time: 3:43p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Split B8Demo.C into B8Demo.c, Logf.c, and Cmds.c. The intention is that
 * Logf.c and Cmds.c can be shared by the B8TST project.
 *
 * *****************  Version 29  *****************
 * User: Jari         Date: 4/25/98    Time: 5:30p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Took out exit on LI reset failure if operated in polling mode
 *
 * *****************  Version 28  *****************
 * User: Jari         Date: 4/25/98    Time: 1:59p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Made base address unsigned long (api changed to support memory mapped
 * boards)
 *
 * *****************  Version 27  *****************
 * User: Hanz         Date: 4/03/98    Time: 3:48p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 *
 * *****************  Version 26  *****************
 * User: Hanz         Date: 4/01/98    Time: 3:51p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * HDLC on Bch1 will not be printed to the log file. Each received  HDLC
 * frame is compared with a "00-01-02------CRC16" frame of length 35
 * including CRC.
 *
 * *****************  Version 25  *****************
 * User: Hanz         Date: 3/26/98    Time: 6:22p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Fixed some compiler warning/errors in MSC51 driver. In API.C we now
 * call otsMemcpy() and otsMalloc() instead of calling memcpy() and
 * malloc() directly.
 *
 * *****************  Version 24  *****************
 * User: Hanz         Date: 3/18/98    Time: 7:04p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Apps/B8Demo
 * Now we exit if Construct or Init driver fails.
 *
 * *****************  Version 23  *****************
 * User: Hanz         Date: 3/18/98    Time: 12:17p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Inplemented the recording functions with Ping Pong buffers (in the same
 * way as playback).
 *
 * *****************  Version 22  *****************
 * User: Hanz         Date: 3/18/98    Time: 8:50a
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Apps/B8Demo
 * Implemented playback in the VXD driver. Increased the sample buffer
 * size to 100*102, which is not tested. With a 512 bytes sample buffer
 * size it works fine.
 *
 * *****************  Version 21  *****************
 * User: Hanz         Date: 3/18/98    Time: 7:42a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Changed the plackback function so that he uses a ping-pong buffer for
 * data transferred from the application to the driver. This works in the
 * BC driver. B8TST runs OK.
 *
 * *****************  Version 20  *****************
 * User: Hanz         Date: 3/17/98    Time: 5:47p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Apps/B8Demo
 * Playback changes.
 *
 * *****************  Version 19  *****************
 * User: Hanz         Date: 3/16/98    Time: 5:54p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD/Apps/B8Demo
 * Made some changes to playback...
 *
 * *****************  Version 18  *****************
 * User: Hanz         Date: 3/16/98    Time: 5:32p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Added support for Aborted HDLC frames on the B-channel. Also added the
 * FCS check. B8TST runs OK. HDLC B-channel frames only tested with a few
 * samples files (3ff, abort, and 3msg).
 *
 * *****************  Version 17  *****************
 * User: Hanz         Date: 3/15/98    Time: 10:43p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Added support for HDLC reception on the B-channels. Not quite
 * complemeted. Need to add two more HDLC fifos in the host driver, and
 * add  the FCS check.
 *
 * *****************  Version 16  *****************
 * User: Hanz         Date: 3/14/98    Time: 5:47p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 *
 * *****************  Version 15  *****************
 * User: Hanz         Date: 3/11/98    Time: 7:58p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Fixed a destination overrun bug in memset().
 *
 * *****************  Version 14  *****************
 * User: Hanz         Date: 3/09/98    Time: 7:49p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Added a prompt for digital gain for every "sound" generating function.
 *
 * *****************  Version 13  *****************
 * User: Hanz         Date: 3/08/98    Time: 8:47p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Changed parameters to bldPhoneOn() so that the microphone and speaker
 * are treated as two separate entities. Started to implement a handset
 * "monitor" functionality where the handset speaker gets both the DSP
 * generated data and the received data from ISAC. This functionality does
 * not work yet. Also implemented separate speaker gain for the handset
 * speaker. Also implemented a "raw" playback/recording option.
 *
 * *****************  Version 12  *****************
 * User: Hanz         Date: 2/21/98    Time: 4:04p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 * Renamed bldResetDsp() to bldRestartDsp(). Removed the law parameter
 * from the api function and added the law parameter to bldResetDsp() and
 * bldConfigureDsp(). I.e. we only set the companding law when we start
 * the DSP program.
 *
 * *****************  Version 11  *****************
 * User: Hanz         Date: 1/29/98    Time: 8:10p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * The inp() and outp() functions are now active.
 *
 * *****************  Version 10  *****************
 * User: Hanz         Date: 1/26/98    Time: 3:20p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Added more ifdef's for B8_LITE. We are now down to 27046 bytes of code
 * size for the Lite driver.
 *
 * *****************  Version 9  *****************
 * User: Hanz         Date: 1/25/98    Time: 10:39p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Made cosmetic changes for release MSC51 driver  0.9.
 *
 * *****************  Version 8  *****************
 * User: Hanz         Date: 1/25/98    Time: 9:44p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Added prompt for noOfSkipSamples in playBack.
 *
 * *****************  Version 7  *****************
 * User: Hanz         Date: 1/25/98    Time: 12:12p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Implemented bldRecordBch() for both B-channels. However, playback
 * BCHAN.REC fails. This needs to be investigated.
 *
 * *****************  Version 6  *****************
 * User: Hanz         Date: 1/24/98    Time: 11:25p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Implemented bldRecordBch(). Now works for both B-channels.
 *
 * *****************  Version 5  *****************
 * User: Hanz         Date: 1/24/98    Time: 8:23p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Implemented a new version of playback where the ISR is sending the data
 * from a buffer supplied by the application.
 *
 * *****************  Version 4  *****************
 * User: Hanz         Date: 1/23/98    Time: 8:58p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * Added handling of new fmType BLD_FM_STAT
 *
 * *****************  Version 3  *****************
 * User: Hanz         Date: 1/23/98    Time: 6:19p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc/Apps/B8Demo
 *
 * *****************  Version 2  *****************
 * User: Hanz         Date: 1/23/98    Time: 3:02p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 * We are now load DSPs now.
 *
 * *****************  Version 1  *****************
 * User: Hanz         Date: 1/23/98    Time: 12:10p
 * Created in $/Products/Bld8-20/Design/Drv/Dos/Msc51/Apps/B8Demo
 *
 * *****************  Version 31  *****************
 * User: Hanz         Date: 1/19/98    Time: 8:27p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 *
 * *****************  Version 30  *****************
 * User: Hanz         Date: 1/16/98    Time: 9:31p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Cleaned up DSP 2 Host Command.
 * Added DSP status message to the status FIFO.
 *
 * *****************  Version 29  *****************
 * User: Hanz         Date: 1/16/98    Time: 3:26p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Implemented new command to allow the host to reset the DSP (make him
 * start over from _c_int00).
 *
 * *****************  Version 28  *****************
 * User: Hanz         Date: 1/04/98    Time: 3:10p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Removed dtmfEncode and dtmfDecode from H2D.H.
 *
 * *****************  Version 27  *****************
 * User: Hanz         Date: 1/01/98    Time: 10:49a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Added some debugging code. Temporary, should be removed in future
 * revisions.
 *
 * *****************  Version 26  *****************
 * User: Hanz         Date: 12/26/97   Time: 8:25p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Changed user interface so that commands are sent to all Lis (Dsps) if
 * the '8'  key is pressed. Also added bldDspCmd() on the '#' key.
 *
 * *****************  Version 25  *****************
 * User: Neil         Date: 12/18/97   Time: 5:10p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Merged with changes in main branch labelled 0.8.3
 *
 * *****************  Version 24  *****************
 * User: Jari         Date: 12/12/97   Time: 7:31p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Added /N option for running all the LIs in NT mode
 *
 * *****************  Version 23  *****************
 * User: Jari         Date: 12/11/97   Time: 12:18p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Implemented debugLog option that allows turning debug printouts to the
 * log file DPRINTFs on and off
 *
 * *****************  Version 22  *****************
 * User: Jari         Date: 12/11/97   Time: 12:02p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Made env global
 *
 * *****************  Version 21  *****************
 * User: Jari         Date: 12/10/97   Time: 7:32p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Added /U command line parameter for automatic line activation.
 * Improved logging and added log file header comment
 *
 * *****************  Version 20  *****************
 * User: Jari         Date: 12/09/97   Time: 3:42p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Made logF a global so that it can be shared with driver when executed
 * in polling mode. Added '*' to flush the log file and '!' to write notes
 * to the log file.
 *
 * *****************  Version 19  *****************
 * User: Jari         Date: 12/09/97   Time: 1:45p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc Sohl Fix
 * Added /P command line switch for running in polling mode.
 *
 * *****************  Version 18  *****************
 * User: Lme          Date: 12/03/97   Time: 9:10p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51 Sohl Fix
 * Merging in essential non-DTMF/FSK fixes from the main branch (as of
 * 0.8)
 *
 * *****************  Version 17  *****************
 * User: Hanz         Date: 11/26/97   Time: 7:00p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Msc51
 *
 * *****************  Version 16  *****************
 * User: Lme          Date: 11/14/97   Time: 7:02p
 * Updated in $/Products/Bld8-20/Design/Drv/NT/Kmd/Apps/Blddemo
 *
 * *****************  Version 15  *****************
 * User: Neil         Date: 11/07/97   Time: 11:24a
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * Added apiSinewaveOnBch( ) function which allows the host to set the
 * sinewave frequency and amplitude.
 *
 *
 * *****************  Version 14  *****************
 * User: Lme          Date: 10/31/97   Time: 8:20p
 * Updated in $/Products/Bld8-20/Design/Drv/NT/Kmd/Apps/Blddemo
 *
 * *****************  Version 13  *****************
 * User: Jari         Date: 10/23/97   Time: 4:41p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Converted int's to short's to make porting between 16 and 32-bit
 * platforms easier.
 *
 * *****************  Version 12  *****************
 * User: Jari         Date: 10/21/97   Time: 6:17p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 *
 * *****************  Version 11  *****************
 * User: Jari         Date: 10/21/97   Time: 2:01p
 * Updated in $/Products/Bld8-20/Design/Drv/W95/VxD
 * Took out direct calls to HW and extern references to global data within
 * the driver.
 *
 * *****************  Version 10  *****************
 * User: Jari         Date: 10/18/97   Time: 12:26p
 * Updated in $/Products/Bld8-20/Design/Drv/Dos/Bc
 * bldConstructDriver() split into bldConstructDriver() and
 * bldInitDriver()
 *
 */

/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 1994-1997


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*--------------------------------------------------------------------------*/
