/* $Workfile: CMDS.C $       $Revision: 3 $      $Modtime: 6/24/98 5:48p $  */
/*------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1994-1997

  DESCRIPTION

    Commands which demonstrates the use of the API for
    the Balder-8S 2.0 board.

\*--------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include <ctype.h>

#include "balder.h"              /* Balder Standard Application Programming Interface */
#include "bldsys.h"

#include "logf.h"
#include "cmds.h"


/*--------------------------------------------------------------------------*\
  MACROS
\*--------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------*\
  TYPE DEFINITIONS
\*---------------------------------------------------------------------------*/

typedef struct {
  unsigned char data[256];
  short len;
} FskMsg;


/*--------------------------------------------------------------------------*\
  PRIVATE VARIABLES
\*--------------------------------------------------------------------------*/
static BldRc layer1Status[LI_MAX + 1];
static FskMsg fskMsg;

static long hdlcFramesMatches = 0;


/*--------------------------------------------------------------------------*\
  PRIVATE FUNCTION PROTOTYPES
\*--------------------------------------------------------------------------*/
static short          cmdsIdleFunction(void);
static void           cmdsDemoDelay(long milliseconds);
static BldDigGainType cmdsPromptForDigGain(void);
static void           cmdsResetHardware(void);


void cmdsFromKeyboard(
  void
)
{
  short liNo;
  short firstLiNo = 0;
  short lastLiNo = 0;
  short idleEnabled = TRUE;

  short i;
  BldRc rc, rc1, rc2;
  short           bChannel;
  unsigned char byte;
  unsigned char *fmBuf; // Used for transmitting
  short bigLen = 0;
  short howMany = 1;
  short exitFlag = FALSE;
  static unsigned char bigBuf[300];
  short c;
  short continuous = FALSE;

  while (!exitFlag) {

    if (idleEnabled) {
      cmdsIdleFunction();  // Read from D-channels
    }

    if (continuous) {

      for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
        for (i = 0; i < 100; i++) {
          fmBuf = (unsigned char *) "\x02\xFF\x03\x08\x01\x77\x05\x04\x03\x80\x90\xA2\x18\x01\x82\x34\x01\x40\x3B\x02\x81\x80\x96\x7B\x01\x83";
          if ((rc = bldWriteLi(liNo, fmBuf, 26)) != BLD_SUCCESS) {
            logBldRc(rc);
          } else {
            //logf("Li %d: Sending Setup, 02 FF 03 08 01 77 05 04 03 80 90 A2 18 01 82 34 01 40 3B 02 81 80 96 7B 01 83   \n", liNo);
          }
        }
      }
    }

    if (kbhit()) {
      c = getch();
      logf("\n'%c' Pressed ...\n", c);
      switch (c) {

      case '?':
        logf("Cmd  Synopsis (\'*\' means that the command will only operate one Li)\n", LI_MAX+1);
        logf("===  ========\n");
        logf(" ?   Help text\n");
        logf(" 0-7 Select access, this access is then used for subsequent commands\n");
        logf(" 8   All accesses are selected for subsequent commands\n");
        logf(" a   Activate Li\n");
        logf(" b   Read B-channel byte from Li (and Bch)\n");
        logf(" c   Configure DTMF Detector\n");
        logf(" d   Deactivate Li\n");
        logf(" e   Configure Silence Detector\n");
        logf(" f   HDLC on B-channel\n");
        logf(" g   Set Global Digital Gain (for all transmitted samples)\n");
        logf(" h   Record B-channel samples to a file\n");
        logf(" i * Idle function enabled\n");
        logf(" I * Idle function disabled\n");
        logf(" k   Custom Sinewave on B-channel\n");
        logf(" l   Load DSP Application\n");
        logf(" m   Generate continous DTMF tone\n");
        logf(" n   Turn off tone generation on a B-channel (Send 0)\n");
        logf(" o   Generate tone on a B-channel\n");
        logf(" p   Loop Bchannel seen from interface side\n");
        logf(" q   Quit the program\n");
        logf(" r   Reset all Hw and layer 1 on the board\n");
        logf(" s   Display status (L1, DTMF, Silence, FSK)\n");
        logf(" t   Send TEI request: FC FF 03 0F 29 6B 01 FF\n");
        logf(" u   DTMF dial mode (send DTMF burst)\n");
        logf(" v   Sending a long nonsense message\n");
        logf(" w   Sending two long nonsense messages back-to-back\n");
        logf(" x   Turn byte sending off for a B-channel (Send 0x7F)\n");
        logf(" y   Send Command to DSPs\n");
        logf(" z   Send continuous bytes on a B-channel\n");
        logf(" + * Enter total msg Length to be transmitted when 'v' or 'w' is pressed:\n");
        logf(" - * Enter how many messages to be sent when 'v' is pressed\n");
        logf(" @   Enable test mode, only a reset disables\n");
        logf(" {   Connect handset on a B-channel\n");
        logf(" }   Disconnect handset\n");
        logf(" |   Disconnect terminating resistor\n");
        logf(" \\   Connect terminating resistor\n");
        logf(" %% * Read from IO-port\n");
        logf(" ^ * Write to IO-port\n");
        logf(" &   Playback (send) a file of B-channel samples\n");
        logf(" @   Tone test. Turn on and detect constant DTMF 5 for 100 times\n");
        logf(" * * Flush the log file\n");
        logf(" =   Configure FSK Detector\n");
        logf(" #   Send command to DSP\n");
        logf(" !   Restart DSP program\n");
        break;

      case 'a':
        /* Layer 1 Activate Request */
        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Activating\n", liNo);
          if ((rc = bldActivate(liNo, 500L)) != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;

      case 'b':
        {
          short bchBytes[LI_MAX + 1][2];
          short liNo;

          /* Read a byte from the two B-channels and print it */
          for (i= 1; i <= 2; i++) {
            for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
              bchBytes[liNo][i-1] = 256; /* Error value */
              rc = bldByteReadBch(liNo, i, &byte); // Read a Bch byte
              switch (rc) {
              case BLD_SUCCESS:
                bchBytes[liNo][i-1] = byte;
                break;
              case BLD_TIMEOUT:
                logf("Timeout. Could not read a B-channel %d byte\n", i);
                break;
              default:
                logf("Internal Error: Unknown return code form bldByteReadBch()\n");
                break;
              }
            }
          }

          logf("      Bch1 Bch2   B-channel Bytes: \n");
          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d:  %.2X, %.2X  \n", liNo, bchBytes[liNo][0], bchBytes[liNo][1]);
          }
        }
        break;


      case 'c':
        {
          /* Configure DTMF Detector */
          short storeInFifo;
          short detectorEnabled;

          logf("Configure DTMF Detector for a B-channel.\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("DTMF Detector Enabled: (y/n)");
          detectorEnabled = (getch() == 'y' ? 1 : 0);
          logf("%c\n", detectorEnabled == 1 ? 'y' : 'n');

          logf("Store DTMF Digits in Fifo: (y/n)");
          storeInFifo = (getch() == 'y' ? 1 : 0);
          logf("%c\n", storeInFifo == 1 ? 'y' : 'n');

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: %s DTMF Detector on b-channel %d\n", liNo, detectorEnabled ? "Enabling" : "Disabling", bChannel);
            if ((rc = bldConfigureDtmfDetector(liNo, bChannel, detectorEnabled, storeInFifo)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }

        }
        break;



      case 'd':
        /* Layer 1 Deactivate Request */
        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Deactivating\n", liNo);
          bldDeactivate(liNo);
        }
        break;


      case 'e':
        {
          /* Configure Silence Detector */
          short silenceDuration;
          short thresHold;
          short detectorEnabled;

          logf("Configure Silence Detector for a B-channel.\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("Silence Detector Enabled: (y/n)");
          detectorEnabled = (getch() == 'y' ? 1 : 0);
          logf("%c\n", detectorEnabled == 1 ? 'y' : 'n');

          logf("Duration time (in 12.75ms increments): ");
          scanf("%d", &silenceDuration);

          logf("Silence ThresHold: ");
          scanf("%d", &thresHold);


          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: %s Silence Detector on b-channel %d\n", liNo, detectorEnabled ? "Enabling" : "Disabling", bChannel);
            if ((rc = bldConfigureSilenceDetector(liNo, bChannel, detectorEnabled, silenceDuration, thresHold)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }

        }
        break;


#ifndef B8_LITE
      case 'f':
        {
          short includeFcs;
          BldFrameFillType filler;

          /* Run HDLC on a B-channel */

          logf("HDLC on an a B-channel\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("Include FCS: (y/n)");
          includeFcs = (getch() == 'y' ? 1 : 0);
          logf("%c\n", includeFcs == 1 ? 'y' : 'n');

          logf("InterFrameFill Character: \n");
          logf("0: BLD_FFT_FLAGS      (0x7E)  \n");
          logf("1: BLD_FFT_ALL_ONES   (0xFF)  \n");
          filler = getch() - '0';
          logf("%d\n", filler);

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: HDLC on b-channel %d\n", liNo, bChannel);
            if ((rc = bldHdlcOnBch(liNo, bChannel, includeFcs, filler)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;
#endif // !B8_LITE



#ifndef B8_LITE
      case 'g':
        {
          /* Set Digital Gain */
          BldDigGainType digGain;

          logf("Set Digital Gain for sound generated a B-channel.\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("Digital Gain: \n");
          logf("0: DG_0DB                 Full Gain                 \n");
          logf("1: DG_MINUS_3DB           -3dB Gain                 \n");
          logf("2: DG_MINUS_6DB           -6dB Gain                 \n");
          logf("3: DG_MINUS_9DB           -9dB Gain                 \n");
          logf("4: DG_MINUS_12DB          -12dB Gain                \n");
          logf("5: DG_MINUS_15DB          -15dB Gain                \n");
          digGain = getch() - '0';
          logf("%d\n", digGain);

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Digital Gain[%d] on b-channel %d\n", liNo, digGain, bChannel);
            if ((rc = bldDigGainOnBch(liNo, bChannel, digGain)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;
#endif // !B8_LITE

      case 'h':
        {
          char strBuf[80];
          short compandData;
          static short recBch = 1;
          static char recFileName[80];
          static short  recDuration = 5;
          short  noOfBuffers2Record;
          #define REC_SAMPLE_BUFSIZE (4*102) // This is 4*12.75ms = 51ms
          static short recBuf[REC_SAMPLE_BUFSIZE];
          FILE *recF;
          short n;

          /* Record B-channel samples to a file */
          logf("Record B-channel samples to a file.\n");

          logf("B-channel (%d): ", recBch);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            recBch = (short) strtol(strBuf, NULL, 0);
          }

          logf("File Name (%s): ", recFileName);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            strcpy(recFileName, strBuf);
          }

          logf("Compand Data: (y/n)");
          compandData = (getch() == 'y' ? 1 : 0);
          logf("%c\n", compandData == 1 ? 'y' : 'n');

          logf("Recording duration (in seconds) (%d): ", recDuration);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            recDuration = (short) strtol(strBuf, NULL, 0);
          }
          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Recording B-Channel samples to file %s on b-channel %d\n", liNo, recFileName, recBch);

            if ((recF = fopen(recFileName, "wb")) == NULL) {
              logf("Li %d: Error could not open file %s\n", liNo, recFileName);
              break;
            }

            // Initialize the recording
            if ((rc = bldRecordBchInitiate(liNo, recBch, REC_SAMPLE_BUFSIZE, compandData)) != BLD_SUCCESS) {
              logBldRc(rc);
              break;
            } else {
              logf("Recording Initiated\n");
            }

            noOfBuffers2Record = (recDuration * 1000) / (REC_SAMPLE_BUFSIZE/8);
            for (n = 0; n < noOfBuffers2Record; n++) {
              // Wait until the driver has full buffer for us
              while (bldRecordBch(liNo, recBch, recBuf) != BLD_SUCCESS) {
                if (kbhit()) {
                  getch();
                  cmdsExitQuestion();
                }
              }

              logf("Got buffer %d of %d\n", n+1, noOfBuffers2Record);

              // Write the buffer to the file
              fwrite(recBuf, 2, REC_SAMPLE_BUFSIZE, recF);

            }

            // Turn the recording off
            if ((rc = bldRecordBchTerminate(liNo, recBch)) != BLD_SUCCESS) {
              logBldRc(rc);
              break;
            }

            logf("Li %d: Recording complete\n", liNo);

            fclose(recF);

          }
        }
        break;


      case 'i':
        logf("Idle function disabled\n");
        idleEnabled = FALSE;
        break;

      case 'I':
        logf("Idle function enabled\n");
        idleEnabled = TRUE;
        break;

#if 0
      case 'j':
        logf("Board interrupts disabled\n");
        disableBoardIntr(0);
        break;

      case 'J':
        logf("Board interrupts enabled\n");
        enableBoardInt(0);
        break;
#endif


#ifndef B8_LITE
      case 'k':
        {
          /* Custom Sinewave on a B-channel */
          char strBuf[14];
          short coeff;
          short y1;
          short y2;
          BldDigGainType digGain;

          logf("Generate sinewave on a B-channel.\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("Coefficient: ");
          gets(strBuf);
          coeff = (short) strtol(strBuf, NULL, 0);

          logf("Initial y[n-1]: ");
          gets(strBuf);
          y1 = (short) strtol(strBuf, NULL, 0);

          logf("Initial y[n-2]: ");
          gets(strBuf);
          y2 = (short) strtol(strBuf, NULL, 0);

          digGain = cmdsPromptForDigGain();

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Sinewave on b-channel %d\n", liNo, bChannel);
            if ((rc = bldSinewaveOnBch(liNo, bChannel, coeff, y1, y2, digGain)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;
#endif // !B8_LITE


#ifndef B8_LITE
      case 'l':
        {
          char appFile[80];
          char entryPoint[20];
          short liNo;
          BldLawType law;

          logf("Loads a DSP application\n");
          logf("Application filename?  ");
          gets(appFile);

          logf("Entry point:\n");
          logf("  1: _c_int00\n");
          logf("  2: start\n");
          logf("  3: other...\n");

          switch (getch()) {
          case '1':
            strcpy(entryPoint, "_c_int00");
            break;
          case '2':
            strcpy(entryPoint, "start");
            break;
          default:
            logf("Enter entry point:  ");
            gets(entryPoint);
            break;
          }
          logf("\n");

          logf("Which Companding Law: \n");
          logf("0: U-LAW  \n");
          logf("1: A-LAW  \n");
          law = getch() - '0';


          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            if ((rc = bldConfigureDsp(liNo, appFile, entryPoint, law)) != BLD_SUCCESS) {
              logBldRc(rc);
            } else {
              logf("Load was a success! Running the application\n");
            }
          }
        }
        break;
#endif // !B8_LITE

      case 'm':
        {
          char bNumber[2];
          short digit;
          BldDigGainType digGain;

          /* Set DTMF Tone on an a B-channel */
          logf("Set DTMF Tone on an a B-channel\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);
          logf("Digit: ");
          digit = getch();
          logf("%c\n", digit);

          digGain = cmdsPromptForDigGain();

          bNumber[0] = (char)digit;
          bNumber[1] = '\0';
          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Sending a constant DTMF tone %s on b-channel %d\n", liNo, bNumber, bChannel);
            if ((rc = bldDtmfOnBch(liNo, bChannel, bNumber, 0, 0, digGain)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;


      case 'n':
        /* Set Tone off a B-channel */
        logf("Turn Tone off for a B-channel\n", liNo);
        logf("B-channel: ");
        bChannel = getch() - '0';
        logf("%d\n", bChannel);


        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Turning off tone for b-channel %d\n", liNo, bChannel);
          if ((rc = bldToneOffBch(liNo, bChannel)) != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;


      case 'o':
        {
          /* Set Tone on an a B-channel */
          BldToneType toneType;
          BldDigGainType digGain;

          logf("Generate Tone on an a B-channel\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("Tone Type: \n");
          logf("0: TT_SINEWAVE,      A 1kHz sinewave                \n");
          logf("1: TT_SAWTOOTH,      Clicking sound                 \n");
          logf("2: TT_SWEEPING_SINE, A sinewave varying in frequency\n");
          logf("3: TT_NOISE          Random noise                   \n");
          logf("4: TT_BYTE_VARIABLE, Sequential bytes               \n");
          toneType = getch() - '0';
          logf("%d\n", toneType);

          digGain = cmdsPromptForDigGain();

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Tone[%d] on b-channel %d\n", liNo, toneType, bChannel);
            if ((rc = bldToneOnBch(liNo, bChannel, toneType, digGain)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;


#if 0
      case 'p':
        /* Bch loop */
        {
          logf("Bch loop\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            if ((rc = apiBchLoopOn(liNo, bChannel)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;
#endif


      case 'q':
      case 'Q':
        /* Quit program */
        exitFlag = TRUE;
        break;

      case 'r':
        /* Reset all HDLCs */
        logf("Resetting All HDLCs on the board (and Layer 1)\n");
        cmdsResetHardware();
        break;

      case 's':
        {
          /* Check status Li, DTMF, Silence, FSK */
          char dtmfDigit;

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {

            rc = bldGetStatusLi(liNo);
            logf("Li %d: Layer1 status = %s\n", liNo, (rc == BLD_L1_DOWN) ? "Down" : "OK");

            rc = bldGetStatusDtmf(liNo, 1, &dtmfDigit);
            if (rc == BLD_TONE || rc == BLD_NO_TONE) {
              logf("Li %d: DTMF Bch1 status = %s %c\n", liNo, (rc == BLD_TONE) ? "Digit" : "No Digit", dtmfDigit);
            } else {
              logBldRc(rc);
            }

            rc = bldGetStatusDtmf(liNo, 2, &dtmfDigit);
            if (rc == BLD_TONE || rc == BLD_NO_TONE) {
              logf("Li %d: DTMF Bch2 status = %s %c\n", liNo, (rc == BLD_TONE) ? "Digit" : "No Digit", dtmfDigit);
            } else {
              logBldRc(rc);
            }

            rc = bldGetStatusSilence(liNo, 1);
            if (rc == BLD_TONE || rc == BLD_NO_TONE) {
              logf("Li %d: Silence Bch1 status = %s \n", liNo, (rc == BLD_TONE) ? "Non-Silence" : "Silence");
            } else {
              logBldRc(rc);
            }

            rc = bldGetStatusSilence(liNo, 2);
            if (rc == BLD_TONE || rc == BLD_NO_TONE) {
              logf("Li %d: Silence Bch2 status = %s \n", liNo, (rc == BLD_TONE) ? "Non-Silence" : "Silence");
            } else {
              logBldRc(rc);
            }
          }

          logf("hdlcFramesMatches = %ld\n", hdlcFramesMatches);
          hdlcFramesMatches = 0;
        }
        break;


      case 'S':
        {
          /* Multiple DTMF checks */
          char expectedDtmfDigit;
          char dtmfDigit;
          char strBuf[12];
          char digitBuf[12];
          long noOfChecks;
          long n;
          BldDigGainType digGain;

          logf("Check DTMF Status (a number of times)\n");

          logf("B-channel: ");
          gets(strBuf);
          bChannel = (short) strtol(strBuf, NULL, 0);

          #if 0
          logf("Digit: ");
          gets(strBuf);
          expectedDtmfDigit = strBuf[0];
          #endif

          logf("Number of Checks (10ms delay between each check): ");
          gets(strBuf);
          noOfChecks = strtol(strBuf, NULL, 0);

          digGain = cmdsPromptForDigGain();

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            for (expectedDtmfDigit = '0'; expectedDtmfDigit <= '9'; expectedDtmfDigit++) {
              digitBuf[0] = expectedDtmfDigit;
              digitBuf[1] = '\0';

              logf("Li %d: Sending a constant DTMF tone %s on b-channel %d\n", liNo + 1, digitBuf, bChannel);
              if ((rc = bldDtmfOnBch((short)(liNo+1), bChannel, digitBuf, 0, 0, digGain)) != BLD_SUCCESS) {
                logBldRc(rc);
              }

              cmdsDemoDelay(30L);

              for (n = 0; n < noOfChecks; n++) {
                if (n % 10 == 0) {
                  logf("Li %d: DTMF Check number = %ld\n", liNo, n);
                }

                rc = bldGetStatusDtmf(liNo, bChannel, &dtmfDigit);
                if (rc == BLD_TONE) {
                  if (dtmfDigit != expectedDtmfDigit) {
                    logf("Li %d: Error, Detected digit %c, Expected digit %c (Check number %ld)\n", liNo, dtmfDigit, expectedDtmfDigit, n);
                    break;
                  }
                } else if (rc == BLD_NO_TONE) {
                  logf("Li %d: Error, Did not detect a DTMF tone (Check number %ld)\n", liNo, n);
                  break;
                } else {
                  logBldRc(rc);
                  break;
                }

                cmdsDemoDelay(10L);
              }

            }

          }
          logf("Li %d: DTMF Check complete\n", liNo);
        }
        break;


      case 't':
        /* Send a TEI message */
        fmBuf = (unsigned char *) "\xFC\xFF\x03\x0F\x29\x6B\x01\xFF";
        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          if ((rc = bldWriteLi(liNo, fmBuf, 8)) != BLD_SUCCESS) {
            logBldRc(rc);
          } else {
            logf("Li %d: Sending TEI request:  FC FF 03 0F 29 6B 01 FF  \n", liNo);
          }
        }
        break;


      case 'T':
        /* Test cmdsDemoDelay() */
        {
          short n;

          for (n = 0; n < 10; n++) {
            cmdsDemoDelay(1000L);
            logf("%d second\n", n);
          }
        }
        break;



      case 'u':
        {
          static short durationTime = 5;
          static short pauseTime = 5;
          static char bNumber[128];
          static short bCh = 1;
          char strBuf[128];
          BldDigGainType digGain;

          logf("Send DTMF Burst (a string of DTMF digits)\n");
          logf("Dial mode\n");

          logf("Phone number (%s): ", bNumber);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            strcpy(bNumber, strBuf);
          }

          logf("B-channel (%d): ", bCh);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            bCh = (short) strtol(strBuf, NULL, 0);
          }

          logf("Duration time in 12.75ms increments (%d): ", durationTime);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            durationTime = (short) strtol(strBuf, NULL, 0);
          }

          logf("Pause time in 12.75ms increments (%d): ", pauseTime);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            pauseTime = (short) strtol(strBuf, NULL, 0);
          }

          digGain = cmdsPromptForDigGain();

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Calling %s on B-channel %d\n", liNo, bNumber, bCh);
            if ((rc = bldDtmfOnBch(liNo, bCh, bNumber, durationTime, pauseTime, digGain)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;



      case 'v':
        /* Send long nonsense message ('+') */

        for (i = 0; i < bigLen; i++) {
          bigBuf[i] = (unsigned char) i;
        }

        logf("Sending %d long nonsense messages:\n", howMany);
        for (i = 0; i < bigLen; i++) {
          logf("%02X ", (int) bigBuf[i]);
        }
        logf("\n");

        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          for (i = 0; i < howMany; i ++) {
            if ((rc = bldWriteLi(liNo, bigBuf, bigLen)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;



      case 'w':
        /* Send two back-to-back long nonsense message ('+') */
        memcpy(bigBuf, (unsigned char *) "\x02\xFF\x03\x08\x01\x77\x05", 7);

        for (i = 7; i < bigLen; i++) {
          bigBuf[i] = (unsigned char) ((i + 1) % 10);
        }

        logf("Sending two back-to-back long nonsense message:\n");
        for (i = 0; i < bigLen; i++) {
          logf("%02X ", (int) bigBuf[i]);
        }
        logf("\n");

        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          rc1 = bldWriteLi(liNo, bigBuf, bigLen);
          rc2 = bldWriteLi(liNo, bigBuf, bigLen);

          if (rc1 != BLD_SUCCESS) {
            logBldRc(rc);
          }
          if (rc2 != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;



      case 'x':
        /* Set Byte off a B-channel */
        logf("Turn Byte off for a B-channel\n");
        logf("B-channel: ");
        bChannel = getch() - '0';
        logf("%d\n", bChannel);

        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Turning byte off for b-channel %d\n", liNo, bChannel);
          if ((rc = bldByteOffBch(liNo, bChannel)) != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;


      case 'y':
        /* Send a longer message */
        fmBuf = (unsigned char *) "\x02\x85\x00\x00\x08\x01\x77\x05\x04\x03\x80\x90\xA2\x18\x01\x82\x34\x01\x40\x3B\x02\x81\x80\x96\x7B\x01\x83";

        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Sending Setup, 02 85 00 00 08 01 77 05 04 03 80 90 A2 18 01 82 34 01 40 3B 02 81 80 96 7B 01 83   \n", liNo);
          if ((rc = bldWriteLi(liNo, fmBuf, 27)) != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;



      case 'z':
        /* Set Byte on an a B-channel */

        logf("Set Byte on an a B-channel\n", liNo);
        logf("Byte: ");
        scanf("%X", &byte);
        logf("B-channel: ");
        bChannel = getch() - '0';
        logf("%d\n", bChannel);

        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Writing 0x%.2X on b-channel %d\n", liNo, byte, bChannel);
          if ((rc = bldByteOnBch(liNo, bChannel, byte)) != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;


#if 0
      case 'Z':
        /* Set Byte on an a B-channel */

        logf("Set Byte on an a B-channel\n", liNo);
        logf("Byte: ");
        scanf("%X", &byte);
        logf("B-channel: ");
        bChannel = getch() - '0';
        logf("%d\n", bChannel);

        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {

          for (byte = 0; byte < 102; byte++) {
            logf("Li %d: Writing 0x%.2X on b-channel %d\n", liNo, byte, bChannel);
            if ((rc = bldByteOnBch(liNo, bChannel, byte)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
            delay(500);
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
            cmdsIdleFunction();
          }
        }
        break;
#endif


      case '=':
        {
          /* Configure FSK Detector */
          short detectorEnabled;

          logf("Configure FSK Detector for a B-channel.\n");
          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("FSK Detector Enabled: (y/n)");
          detectorEnabled = (getch() == 'y' ? 1 : 0);
          logf("%c\n", detectorEnabled == 1 ? 'y' : 'n');

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: %s FSK Detector on b-channel %d\n", liNo, detectorEnabled ? "Enabling" : "Disabling", bChannel);
            if ((rc = bldConfigureFskDetector(liNo, bChannel, detectorEnabled)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }

        }
        break;



      case '{':
        {
          short aicNo;
          BldAnalogInSrc inSrc;
          BldAnalogOutSrc outSrc;
          BldSpkrGainType spkrGain;

          /* Connect handset */
          logf("Connect handset\n");

          logf("Aic No: ");
          aicNo = getch() - '0';
          logf("%d\n", aicNo);

          logf("B-channel: ");
          bChannel = getch() - '0';
          logf("%d\n", bChannel);

          logf("Analog Input Source:\n");
          logf("0: AIS_DISCONNECT, Don't use the AIC to drive the B-channel \n");
          logf("1: AIS_MIC,        Use the Handset Microphone as the input source  \n");
          logf("2: AIS_AUX         Use the Aux input on the harmonica as input source  \n");
          inSrc = getch() - '0';
          logf("%c\n", inSrc + '0');

          logf("Speaker Source:\n");
          logf("0: AOS_DISCONNECT, Don't connect the speaker at all               \n");
          logf("1: AOS_RX,         Send received S-Bus samples to the speaker     \n");
          logf("2: AOS_TX,         Send generated samples to the speaker          \n");
          logf("3: AOS_RX_AND_TX   Send both recieved S-bus and generated samples \n");
          outSrc = getch() - '0';
          logf("%c\n", outSrc + '0');


          logf("Speaker Gain:\n");
          logf("0: SG_MINUS_6DB            \n");
          logf("1: SG_MINUS_3DB            \n");
          logf("2: SG_0DB         No gain  \n");
          logf("3: SG_PLUS_3DB             \n");
          logf("4: SG_PLUS_6DB             \n");
          spkrGain = getch() - '0';
          logf("%c\n", spkrGain + '0');


          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Connecting handset\n", liNo);
            if ((rc = bldPhoneOn(liNo, bChannel, aicNo, inSrc, outSrc, DG_MINUS_6DB, spkrGain)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;

      case '}':
        /* Disconnect handset */
        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Disconnect handset (Mask AIC interrupts)\n", liNo);
          if ((rc = bldPhoneOff(liNo)) != BLD_SUCCESS) {
            logBldRc(rc);
          }
        }
        break;


      case '-':
        /* Enter how many messages to be sent when 'v' is pressed */
        logf("Enter how many msgs that should be transmitted when 'v' is pressed: ");
        scanf("%d", &howMany);
        break;

      case '+':
        /* Enter a length for the generated message */
        logf("Enter total msg Length to be transmitted when 'v' is pressed: ");
        scanf("%d", &bigLen);
        break;

#ifndef B8_LITE
      case '%':
        {
          static char readAddrStr[12];
          static short readAddr = 0;

          logf("Read from Address (0x%X): ", readAddr);
          gets(readAddrStr);
          if (readAddrStr[0] != '\0') {
            readAddr = (short) strtol(readAddrStr, NULL, 0);
          }
          logf("0x%X: 0x%.2X\n", readAddr, sysOtsInp(readAddr));
        }
        break;
#endif // !B8_LITE

#ifndef B8_LITE
      case '^':
        {
          static char writeAddrStr[12];
          static char writeValStr[12];
          static short writeAddr = 0;
          static short writeVal = 0;

          logf("Write to Address (0x%X): ", writeAddr);
          gets(writeAddrStr);
          if (writeAddrStr[0] != '\0') {
            writeAddr = (short) strtol(writeAddrStr, NULL, 0);
          }
          logf("Value (0x%.2X): ", writeVal);
          gets(writeValStr);
          if (writeValStr[0] != '\0') {
            writeVal = (short) strtol(writeValStr, NULL, 0);
          }
          sysOtsOutp(writeAddr, writeVal);
          logf("Wrote to 0x%.2X to address 0x%X: \n", writeVal, writeAddr);
        }
        break;
#endif // !B8_LITE


      case '|':
        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Disconnecting Terminating Resistor\n", liNo);
          bldTermLi(liNo, FALSE);
        }
        break;

      case '\\':
        for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
          logf("Li %d: Connecting Terminating Resistor\n", liNo);
          bldTermLi(liNo, TRUE);
        }
        break;


      case '&':
        {
          char strBuf[80];
          short compandData;
          static short plBch = 1;
          static short plNoOfSkipSamples = 0;
          static char plFileName[80];
          #define PL_SAMPLE_BUFSIZE (4*102)
          static short sampleBuf[PL_SAMPLE_BUFSIZE];
          FILE *sampleF;
          short readCount;
          short n;
          BldDigGainType digGain;
          long busyCount = 0;


          /* Playback (Send) B-channel samples from a file */
          logf("Playback (send) B-channel samples from a file.\n");

          logf("B-channel (%d): ", plBch);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            plBch = (short) strtol(strBuf, NULL, 0);
          }

          logf("File Name (%s): ", plFileName);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            strcpy(plFileName, strBuf);
          }

          logf("Number of samples to skip from the beginning of the file (%d): ", plNoOfSkipSamples);
          gets(strBuf);
          if (strcmp(strBuf, "")) {
            plNoOfSkipSamples = (short) strtol(strBuf, NULL, 0);
          }

          logf("Compand Data: (y/n)");
          compandData = (getch() == 'y' ? 1 : 0);
          logf("%c\n", compandData == 1 ? 'y' : 'n');

          digGain = cmdsPromptForDigGain();


          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Playing B-Channel samples from file %s on b-channel %d\n", liNo, plFileName, plBch);

            if ((sampleF = fopen(plFileName, "rb")) == NULL) {
              logf("Li %d: Error could not open file %s\n", liNo, plFileName);
              break;
            }

            for (n = 0; n < plNoOfSkipSamples; n++) {
              fread(sampleBuf, 2, 1, sampleF);
            }

            // Fill the buffer with data from the buffer for the first time
            readCount = fread(sampleBuf, 2, PL_SAMPLE_BUFSIZE, sampleF);
            if (readCount != PL_SAMPLE_BUFSIZE) {
              // We reached EOF on this read so we did not fill the entire sample buffer. So pad it with 0's
              memset(sampleBuf + readCount, 0, (PL_SAMPLE_BUFSIZE - readCount) * sizeof(short));
            }

            // Give the first buffer to the driver and tell him to start sending the data
            if ((rc = bldPlaybackBchInitiate(liNo, plBch, sampleBuf, PL_SAMPLE_BUFSIZE, compandData, digGain)) != BLD_SUCCESS) {
              logBldRc(rc);
              break;
            }

            while (!feof(sampleF)) {

              // Fill the buffer again with data from the file
              readCount = fread(sampleBuf, 2, PL_SAMPLE_BUFSIZE, sampleF);
              if (readCount != PL_SAMPLE_BUFSIZE) {
                // We reached EOF on this read so we did not fill the entire sample buffer. So pad it with 0's
                memset(sampleBuf + readCount, 0, (PL_SAMPLE_BUFSIZE - readCount) * sizeof(short));
              }

              // Loop until the driver has accepted this new buffer
              busyCount = 0;
              while (bldPlaybackBch(liNo, plBch, sampleBuf) != BLD_SUCCESS) {
                busyCount++;
                cmdsIdleFunction();
                if (kbhit()) {
                  getch();
                  cmdsExitQuestion();
                }
              }
              //logf("Busy Count %ld\n", busyCount);
            }

            // Loop until the driver has sent the last buffer
            while (bldPlaybackBchTerminate(liNo, plBch, 0) != BLD_SUCCESS) {
              cmdsIdleFunction();
              if (kbhit()) {
                getch();
                cmdsExitQuestion();
              }
            }

            fclose(sampleF);

          }
        }
        break;


      case '!':
        {
          BldLawType law;

          logf("Restarting DSP program. Which Companding Law: \n");
          logf("0: U-LAW  \n");
          logf("1: A-LAW  \n");
          law = getch() - '0';

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Restarting DSP program with law = %d\n", liNo, law);

            if ((rc = bldRestartDsp(liNo, law)) != BLD_SUCCESS) {
              logBldRc(rc);
            } else {
              logf("Done\n");
            }
          }
        }
        break;


#ifndef B8_LITE
      case '#':
        {
          /* Send command to DSP */
          short dspCmd;
          char strBuf[12];

          logf("Li %d: Send command to DSP.\n", liNo);

          logf("DSP Commands: \n");
          logf("  H2D_NO_CMD              =  0 ,      \n");
          logf("  H2D_HOST_DATA_FILL_COMPLETE =  1    \n");
          logf("  H2D_ULAW_THIS_WORD      =  2 ,      \n");
          logf("  H2D_START_BYTE_CHK      =  3 ,      \n");
          logf("  H2D_FILL_TX_BUF         =  4 ,      \n");
          logf("  H2D_CORRUPT_TX_BUF      =  5 ,      \n");
          logf("  H2D_STOP_BYTE_CHK       =  6 ,      \n");
          logf("  H2D_INIT_AIC            =  7 ,      \n");
          logf("  H2D_DISC_AIC            =  8 ,      \n");
          logf("  H2D_INIT_SILENCE_DET1   =  9 ,      \n");
          logf("  H2D_INIT_SILENCE_DET2   = 10 ,      \n");
          logf("  H2D_WR_DATA_REQUEST     = 11 ,      \n");
          logf("  H2D_RD_DATA_UPDATED     = 12        \n");
          logf("                                      \n");
          logf("                                      \n");
          logf("                                      \n");

          gets(strBuf);
          dspCmd = (short) strtol(strBuf, NULL, 0);

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            logf("Li %d: Dsp Command [%d] \n", liNo, dspCmd);
            if ((rc = bldDspCmd(liNo, dspCmd)) != BLD_SUCCESS) {
              logBldRc(rc);
            }
          }
        }
        break;
#endif // !B8_LITE


      case '@':
        {
          char dtmfDigit;
          short detectorEnabled = 1;
          short storeInFifo = 0;
          short bChannel = 1;
          long delayVal;
          short i;
          char strBuf[12];
          BldDigGainType digGain;

          printf("Tone test\n");

          printf("Give the delay value in milliseconds:\n");
          gets(strBuf);
          delayVal = strtol(strBuf, NULL, 0);

          digGain = cmdsPromptForDigGain();

          for (liNo = firstLiNo; liNo <= lastLiNo; liNo++) {
            for (i = 0; i < 100; i++) {
              logf("Li %d: %s DTMF Detector on b-channel %d\n", liNo, detectorEnabled ? "Enabling" : "Disabling", bChannel);
              if ((rc = bldConfigureDtmfDetector((short)(liNo+1), bChannel, detectorEnabled, storeInFifo)) != BLD_SUCCESS) {
                logBldRc(rc);
              }

              if ((rc = bldDtmfOnBch(liNo, bChannel, "5", 0, 0, digGain)) != BLD_SUCCESS) {
                logBldRc(rc);
              }

              cmdsDemoDelay(delayVal);

              rc = bldGetStatusDtmf((short)(liNo+1), bChannel, &dtmfDigit);
              if (rc == BLD_TONE || rc == BLD_NO_TONE) {
                printf("Li %d: DTMF Bch1 status = %s %c\n", liNo, (rc == BLD_TONE) ? "Digit" : "No Digit", dtmfDigit);
              } else {
                logBldRc(rc);
              }

              // Turn tone off
              if ((rc = bldByteOnBch(liNo, bChannel, 0xFF)) != BLD_SUCCESS) {
                logBldRc(rc);
              }

              cmdsDemoDelay(delayVal);
            }
          }

        }

        break;


#if 0
      case '*':
        logf("Closing And opening the log file\n");
        logf("Closing ...\n");
        fclose(logF);
        if ((logF = fopen(logFileName, "at")) == NULL) {
          printf("Couldn't open logfile %s\n", logFileName);
          exit(-1);
        }
        logf("\nLog file '%s' re-opened ....\n", logFileName);
        break;
#endif



      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
        /* Select LI to act as target for subsequent commands */
        logf("Li %d: Selected\n", (int) c - '0');
        firstLiNo = lastLiNo = c - '0';
        break;
      case '8':
        /* Select all LIs as targets for subsequent commands */
        logf("All Lis are selected\n", (int) c - '0');
        firstLiNo = 0;
        lastLiNo = LI_MAX;
        break;


      default:
        logf("Unknown command. Press '?' for help, 'q' to quit\n");
        break;
      }
    }
  }
}



void cmdsDemoDelay(
/*---------------------------------------------------------------------------*\

  SYNOPSIS

     Halts execution for a specified time period.

\*---------------------------------------------------------------------------*/
  long milliseconds
)
{
  clock_t startTime = clock();
  clock_t now;
  for (;;) {
    now = clock();
    if (now < startTime) {
      // We wrapped (after 50 days)
      return;
    }

    #ifdef MSC51
    if (now > startTime + milliseconds ) {
      return;
    }
    #else
    if (now > startTime + ((milliseconds * CLK_TCK) / 1000) ) {
      return;
    }
    #endif

    if (kbhit()) {
      getch();
      cmdsExitQuestion();
    }

  }
}


void cmdsExitQuestion()
/*--------------------------------------------------------------------------*\

  SYNOPSIS

    Prompts the user if he wants to exit the program.

\*--------------------------------------------------------------------------*/
{
  logf("Do you want to exit? (Y/N)\n");

  while (kbhit()) getch(); // Empty keyboard buffer

  if (toupper(getch()) == 'Y') {
    cmdsExit(1);
  }
}



static short cmdsIdleFunction(
/*--------------------------------------------------------------------------*\

  SYNOPSIS:

    Reads a message from any LI (if a message is available). If more
    than one message is available the oldest message is read.

\*--------------------------------------------------------------------------*/
  void
)
{
  unsigned char *p;
  unsigned short i;
  short liNo;
  short rc = 0;

  #define BUF_SIZE  270    /* Must be at least 270 to accomodate all  */
                           /* .. possible (legal) ISDN messages       */
  static unsigned char msgBuf[BUF_SIZE];
  BldFrameHeader fmHeader;


  if (bldRead(msgBuf, BUF_SIZE, &fmHeader) != BLD_NO_FRAMES) {
    rc = 1;

    #if 0
    if (fmHeader.fmType == BLD_FM_HDLC1) {
      if (fmHeader.fmLength != 35) {
        logf("Mismatch in length for Bch1 HDLC frame\n");
      } else if (memcmp(msgBuf, "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x00\x01\x02\x86\xED", 35)) {
        logf("Mismatch in contents for Bch1 HDLC frame\n");
      } else {
        hdlcFramesMatches++;
      }
      return rc;
    }
    #endif


    logf(">> Li:%d ", (short) fmHeader.liNo);
    logf("#%03d " , (short) fmHeader.fmSeqNo);
    logf("%02d:"  , (short) fmHeader.hour);
    logf("%02d:"  , (short) fmHeader.min);
    logf("%02d."  , (short) fmHeader.sec);
    logf("%02d [" , (short) fmHeader.csec);
    logf("%.2X] " , (short) fmHeader.fmStatus);

    p = msgBuf;

    switch (fmHeader.fmType) {
    case BLD_FM_I: // Only used on auto-mode
      logf("I-frame ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (short) *p++);
      }
      break;
    case BLD_FM_UI: // Only used in auto-mode
      logf("UI-frame ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (short) *p++);
      }
      break;
    case BLD_FM_HDLC:
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (short) *p++);
      }
      break;
    case BLD_FM_HDLC1:
      logf("BCH1 ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (short) *p++);
      }
      break;
    case BLD_FM_HDLC2:
      logf("BCH2 ");
      for (i = 0; i < fmHeader.fmLength; i++) {
        logf("%02X ", (short) *p++);
      }
      break;
    case BLD_FM_STR: // '\0' terminated info string
      logf("%s", msgBuf);
      break;
    case BLD_FM_STAT: // Status code
      #ifdef B8_LITE
      logf("STATUS [%d]", msgBuf[0]);
      #else
      logf("STATUS [%d] %s", msgBuf[0], bldStatus2Str(msgBuf[0]));
      #endif
      break;
    case BLD_FM_DTMF1: // DTMF digit
      logf("DTMF BCH1 Digit %c", msgBuf[0]);
      break;
    case BLD_FM_DTMF2: // DTMF digit
      logf("DTMF BCH2 Digit %c", msgBuf[0]);
      break;
    case BLD_FM_FSK1: // FSK octet Bch1
      logf("FSK BCH1 Octet %.2X", msgBuf[0]);
      fskMsg.data[fskMsg.len++] = msgBuf[0];
      break;
    case BLD_FM_FSK2: // FSK octet Bch2
      logf("FSK BCH2 Octet %.2X", msgBuf[0]);
      fskMsg.data[fskMsg.len++] = msgBuf[0];
      break;
    default:
      logf("Error: Invalid frame type");
      break;
    }
    logf("\n");;
  } else {
    // Don't print this stuff until we have encountered an empty read buffer

    /* Check layer 1 status, print any change (Although deactivation and  */
    /* Activations are endicated by STR messages returned by bldRead()    */
    for (liNo = 0; liNo <= LI_MAX ; liNo++) {
      BldRc rc;
      rc = bldGetStatusLi(liNo);
      if (layer1Status[liNo] != rc) {
        logf("Li:%d Layer 1 status = %s\n", liNo, (rc == BLD_L1_DOWN) ? "Down" : "OK");
        layer1Status[liNo] = rc;
      }
    }
  }

  return rc;
}


/*---------------------------------------------------- Function Description -*\

  SYNOPSIS

    Uninstall the driver.

\*---------------------------------------------------------------------------*/
void cmdsExit(
  short exitCode
)
{
  BldRc rc;
  if ((rc = bldDestructDriver()) != BLD_SUCCESS) {
    logBldRc(rc);
  }

  if (logF) {
    fclose(logF);
  }

  exit(exitCode);
}


static void cmdsResetHardware(
/*---------------------------------------------------- Function Description -*\

  SYNOPSIS

    Resets the Layer 2 and Layer 1 controller/tranceiver for each LI.

\*---------------------------------------------------------------------------*/
  void
)
{
  BldRc rc;
  short liNo;

  for (liNo = 0; liNo <= LI_MAX; liNo++) {
    if ((rc = bldResetLi(liNo)) != BLD_SUCCESS) {
      // Errors
      logBldRc(rc);
    }
  }
}

/*--------------------------------------------------------------------------*\

  SYNOPSIS:

    Propmpt for which digital gain to use.

\*--------------------------------------------------------------------------*/
static BldDigGainType cmdsPromptForDigGain(void)
{
  BldDigGainType digGain;
  logf("Digital Gain: \n");
  logf("0: DG_0DB                 Full Gain                 \n");
  logf("1: DG_MINUS_3DB           -3dB Gain                 \n");
  logf("2: DG_MINUS_6DB           -6dB Gain                 \n");
  logf("3: DG_MINUS_9DB           -9dB Gain                 \n");
  logf("4: DG_MINUS_12DB          -12dB Gain                \n");
  logf("5: DG_MINUS_15DB          -15dB Gain                \n");
  digGain = getch() - '0';
  logf("%d\n", digGain);

  return digGain;
}
